# ALTRepo API
# Copyright (C) 2021  BaseALT Ltd

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from dataclasses import dataclass


@dataclass(frozen=True)
class LookupTables:
    known_branches = [
        "4.0",
        "4.1",
        "5.0",
        "5.1",
        "c6",
        "c7",
        "c7.1",
        "c8",
        "c8.1",
        "c9f1",
        "c9f2",
        "p5",
        "p6",
        "p7",
        "p8",
        "p9",
        "p9_mipsel",
        "p9_e2k",
        "p10",
        "sisyphus",
        "sisyphus_mipsel",
        "sisyphus_riscv64",
        "sisyphus_e2k",
        "t6",
        "t7",
    ]
    taskless_branches = ["p9_mipsel", "sisyphus_mipsel", "sisyphus_riscv64", "sisyphus_e2k"]
    no_downloads_branches = ["sisyphus_e2k", "p9_e2k"]
    known_archs = [
        "x86_64",
        "noarch",
        "x86_64-i586",
        "armh",
        "arm",
        "i586",
        "pentium4",
        "athlon",
        "pentium3",
        "i686",
        "armv5tel",
        "k6",
        "aarch64",
        "ppc64le",
        "e2kv5",
        "e2kv4",
        "e2k",
        "mipsel",
        "riscv64",
    ]
    default_archs = ["x86_64", "i586", "aarch64", "armh", "ppc64le", "noarch"]
    package_params = [
        "pkg_cs",
        "pkg_packager",
        "pkg_packager_email",
        "pkg_name",
        "pkg_arch",
        "pkg_version",
        "pkg_release",
        "pkg_epoch",
        "pkg_serial_",
        "pkg_buildtime",
        "pkg_buildhost",
        "pkg_size",
        "pkg_archivesize",
        "pkg_rpmversion",
        "pkg_cookie",
        "pkg_disttag",
        "pkg_sourcerpm",
        "pkg_filename",
        "pkg_filesize",
        "pkg_srcrpm_hash",
        "pkg_summary",
        "pkg_description",
        "pkg_distribution",
        "pkg_vendor",
        "pkg_gif",
        "pkg_xpm",
        "pkg_license",
        "pkg_group_",
        "pkg_url",
        "pkg_os",
        "pkg_prein",
        "pkg_postin",
        "pkg_preun",
        "pkg_postun",
        "pkg_icon",
        "pkg_preinprog",
        "pkg_postinprog",
        "pkg_preunprog",
        "pkg_postunprog",
        "pkg_buildarchs",
        "pkg_verifyscript",
        "pkg_verifyscriptprog",
        "pkg_prefixes",
        "pkg_instprefixes",
        "pkg_optflags",
        "pkg_disturl",
        "pkg_payloadformat",
        "pkg_payloadcompressor",
        "pkg_payloadflags",
        "pkg_platform",
        "pkg_sourcepackage",
    ]
    pkg_groups = [
        "Accessibility",
        "Archiving/Backup",
        "Archiving/Cd burning",
        "Archiving/Compression",
        "Archiving/Other",
        "Books/Computer books",
        "Books/Faqs",
        "Books/Howtos",
        "Books/Other",
        "Communications",
        "Databases",
        "Development/C",
        "Development/C++",
        "Development/Databases",
        "Development/Debug",
        "Development/Debuggers",
        "Development/Documentation",
        "Development/Erlang",
        "Development/Functional",
        "Development/GNOME and GTK+",
        "Development/Haskell",
        "Development/Java",
        "Development/KDE and QT",
        "Development/Kernel",
        "Development/Lisp",
        "Development/ML",
        "Development/Objective-C",
        "Development/Other",
        "Development/Perl",
        "Development/Python",
        "Development/Python3",
        "Development/Ruby",
        "Development/Scheme",
        "Development/Tcl",
        "Development/Tools",
        "Documentation",
        "Editors",
        "Education",
        "Emulators",
        "Engineering",
        "File tools",
        "Games/Adventure",
        "Games/Arcade",
        "Games/Boards",
        "Games/Cards",
        "Games/Educational",
        "Games/Other",
        "Games/Puzzles",
        "Games/Sports",
        "Games/Strategy",
        "Graphical desktop/Enlightenment",
        "Graphical desktop/FVWM based",
        "Graphical desktop/GNOME",
        "Graphical desktop/GNUstep",
        "Graphical desktop/Icewm",
        "Graphical desktop/KDE",
        "Graphical desktop/MATE",
        "Graphical desktop/Motif",
        "Graphical desktop/Other",
        "Graphical desktop/Sawfish",
        "Graphical desktop/Sugar",
        "Graphical desktop/Window Maker",
        "Graphical desktop/XFce",
        "Graphics",
        "Monitoring",
        "Networking/Chat",
        "Networking/DNS",
        "Networking/FTN",
        "Networking/File transfer",
        "Networking/IRC",
        "Networking/Instant messaging",
        "Networking/Mail",
        "Networking/News",
        "Networking/Other",
        "Networking/Remote access",
        "Networking/WWW",
        "Office",
        "Other",
        "Publishing",
        "Sciences/Astronomy",
        "Sciences/Biology",
        "Sciences/Chemistry",
        "Sciences/Computer science",
        "Sciences/Geosciences",
        "Sciences/Mathematics",
        "Sciences/Medicine",
        "Sciences/Other",
        "Sciences/Physics",
        "Security/Antivirus",
        "Security/Networking",
        "Shells",
        "Sound",
        "System/Base",
        "System/Configuration/Boot and Init",
        "System/Configuration/Hardware",
        "System/Configuration/Networking",
        "System/Configuration/Other",
        "System/Configuration/Packaging",
        "System/Configuration/Printing",
        "System/Fonts/Console",
        "System/Fonts/True type",
        "System/Fonts/Type1",
        "System/Fonts/X11 bitmap",
        "System/Internationalization",
        "System/Kernel and hardware",
        "System/Legacy libraries",
        "System/Libraries",
        "System/Servers",
        "System/Servers/ZProducts",
        "System/X11",
        "System/XFree86",
        "Terminals",
        "Text tools",
        "Toys",
        "Video",
    ]
    known_beehive_branches = [
        "sisyphus",
        "p10",
        "p9",
    ]
    known_beehive_archs = [
        "i586",
        "x86_64",
    ]

    gitalt_base = "https://git.altlinux.org"
    beehive_base = "https://git.altlinux.org/beehive"
    gitalt_tasks_base = "https://git.altlinux.org/tasks"
    packages_base = "https://packages.altlinux.org/en"
    bugzilla_base = "https://bugzilla.altlinux.org"

    rpmsense_flags = [
        "RPMSENSE_ANY",
        "RPMSENSE_SERIAL",
        "RPMSENSE_LESS",
        "RPMSENSE_GREATER",
        "RPMSENSE_EQUAL",
        "RPMSENSE_PROVIDES",
        "RPMSENSE_CONFLICTS",
        "RPMSENSE_PREREQ",
        "RPMSENSE_OBSOLETES",
        "RPMSENSE_INTERP",
        "RPMSENSE_SCRIPT_PRE",
        "RPMSENSE_SCRIPT_POST",
        "RPMSENSE_SCRIPT_PREUN",
        "RPMSENSE_SCRIPT_POSTUN",
        "RPMSENSE_SCRIPT_VERIFY",
        "RPMSENSE_FIND_REQUIRES",
        "RPMSENSE_FIND_PROVIDES",
        "RPMSENSE_TRIGGERIN",
        "RPMSENSE_TRIGGERUN",
        "RPMSENSE_TRIGGERPOSTUN",
        "___SENSE_MULTILIB",
        "RPMSENSE_SCRIPT_PREP",
        "RPMSENSE_SCRIPT_BUILD",
        "RPMSENSE_SCRIPT_INSTALL",
        "RPMSENSE_SCRIPT_CLEAN",
        "RPMSENSE_RPMLIB",
        "RPMSENSE_TRIGGERPREIN",
        "RPMSENSE_KEYRING",
    ]


lut = LookupTables()
